@@ -650,7 +650,6 @@ check_interval = 0
     image = ""
     namespace = ""
     namespace_overwrite_allowed = ""
-    privileged = false
     service_account_overwrite_allowed = ""
     pod_annotations_overwrite_allowed = ""
     [runners.kubernetes.affinity]
@@ -662,6 +661,7 @@ check_interval = 0
         mount_path = "/path/to/empty_dir"
         medium = "Memory"
     [runners.kubernetes.dns_config]
+    [runners.kubernetes.container_lifecycle]
 `,
 		},
 		"incorrect config template merge": {
