@@ -16,6 +16,8 @@ import (
 	"github.com/docker/go-units"
 	"github.com/sirupsen/logrus"
 	api "k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/util/intstr"
 
 	"gitlab.com/gitlab-org/gitlab-runner/helpers"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker"
@@ -29,6 +31,8 @@ import (
 type DockerPullPolicy string
 type DockerSysCtls map[string]string
 
+type KubernetesHookHandlerType string
+
 const (
 	PullPolicyAlways       = "always"
 	PullPolicyNever        = "never"
@@ -184,18 +188,20 @@ type DockerMachineAutoscaling struct {
 
 //nolint:lll
 type ParallelsConfig struct {
-	BaseName         string `toml:"base_name" json:"base_name" long:"base-name" env:"PARALLELS_BASE_NAME" description:"VM name to be used"`
-	TemplateName     string `toml:"template_name,omitempty" json:"template_name" long:"template-name" env:"PARALLELS_TEMPLATE_NAME" description:"VM template to be created"`
-	DisableSnapshots bool   `toml:"disable_snapshots,omitzero" json:"disable_snapshots" long:"disable-snapshots" env:"PARALLELS_DISABLE_SNAPSHOTS" description:"Disable snapshoting to speedup VM creation"`
-	TimeServer       string `toml:"time_server,omitempty" json:"time_server" long:"time-server" env:"PARALLELS_TIME_SERVER" description:"Timeserver to sync the guests time from. Defaults to time.apple.com"`
+	BaseName         string   `toml:"base_name" json:"base_name" long:"base-name" env:"PARALLELS_BASE_NAME" description:"VM name to be used"`
+	TemplateName     string   `toml:"template_name,omitempty" json:"template_name" long:"template-name" env:"PARALLELS_TEMPLATE_NAME" description:"VM template to be created"`
+	DisableSnapshots bool     `toml:"disable_snapshots,omitzero" json:"disable_snapshots" long:"disable-snapshots" env:"PARALLELS_DISABLE_SNAPSHOTS" description:"Disable snapshoting to speedup VM creation"`
+	TimeServer       string   `toml:"time_server,omitempty" json:"time_server" long:"time-server" env:"PARALLELS_TIME_SERVER" description:"Timeserver to sync the guests time from. Defaults to time.apple.com"`
+	AllowedImages    []string `toml:"allowed_images,omitempty" json:"allowed_images" long:"allowed-images" env:"PARALLELS_ALLOWED_IMAGES" description:"Image (base_name) allowlist"`
 }
 
 //nolint:lll
 type VirtualBoxConfig struct {
-	BaseName         string `toml:"base_name" json:"base_name" long:"base-name" env:"VIRTUALBOX_BASE_NAME" description:"VM name to be used"`
-	BaseSnapshot     string `toml:"base_snapshot,omitempty" json:"base_snapshot" long:"base-snapshot" env:"VIRTUALBOX_BASE_SNAPSHOT" description:"Name or UUID of a specific VM snapshot to clone"`
-	BaseFolder       string `toml:"base_folder" json:"base_folder" long:"base-folder" env:"VIRTUALBOX_BASE_FOLDER" description:"Folder in which to save the new VM. If empty, uses VirtualBox default"`
-	DisableSnapshots bool   `toml:"disable_snapshots,omitzero" json:"disable_snapshots" long:"disable-snapshots" env:"VIRTUALBOX_DISABLE_SNAPSHOTS" description:"Disable snapshoting to speedup VM creation"`
+	BaseName         string   `toml:"base_name" json:"base_name" long:"base-name" env:"VIRTUALBOX_BASE_NAME" description:"VM name to be used"`
+	BaseSnapshot     string   `toml:"base_snapshot,omitempty" json:"base_snapshot" long:"base-snapshot" env:"VIRTUALBOX_BASE_SNAPSHOT" description:"Name or UUID of a specific VM snapshot to clone"`
+	BaseFolder       string   `toml:"base_folder" json:"base_folder" long:"base-folder" env:"VIRTUALBOX_BASE_FOLDER" description:"Folder in which to save the new VM. If empty, uses VirtualBox default"`
+	DisableSnapshots bool     `toml:"disable_snapshots,omitzero" json:"disable_snapshots" long:"disable-snapshots" env:"VIRTUALBOX_DISABLE_SNAPSHOTS" description:"Disable snapshoting to speedup VM creation"`
+	AllowedImages    []string `toml:"allowed_images,omitempty" json:"allowed_images" long:"allowed-images" env:"VIRTUALBOX_ALLOWED_IMAGES" description:"Image allowlist"`
 }
 
 //nolint:lll
@@ -284,7 +290,7 @@ type KubernetesConfig struct {
 	Image                                             string                       `toml:"image" json:"image" long:"image" env:"KUBERNETES_IMAGE" description:"Default docker image to use for builds when none is specified"`
 	Namespace                                         string                       `toml:"namespace" json:"namespace" long:"namespace" env:"KUBERNETES_NAMESPACE" description:"Namespace to run Kubernetes jobs in"`
 	NamespaceOverwriteAllowed                         string                       `toml:"namespace_overwrite_allowed" json:"namespace_overwrite_allowed" long:"namespace_overwrite_allowed" env:"KUBERNETES_NAMESPACE_OVERWRITE_ALLOWED" description:"Regex to validate 'KUBERNETES_NAMESPACE_OVERWRITE' value"`
-	Privileged                                        bool                         `toml:"privileged,omitzero" json:"privileged" long:"privileged" env:"KUBERNETES_PRIVILEGED" description:"Run all containers with the privileged flag enabled"`
+	Privileged                                        *bool                        `toml:"privileged,omitzero" json:"privileged" long:"privileged" env:"KUBERNETES_PRIVILEGED" description:"Run all containers with the privileged flag enabled"`
 	AllowPrivilegeEscalation                          *bool                        `toml:"allow_privilege_escalation,omitzero" json:"allow_privilege_escalation" long:"allow-privilege-escalation" env:"KUBERNETES_ALLOW_PRIVILEGE_ESCALATION" description:"Run all containers with the security context allowPrivilegeEscalation flag enabled. When empty, it does not define the allowPrivilegeEscalation flag in the container SecurityContext and allows Kubernetes to use the default privilege escalation behavior."`
 	CPULimit                                          string                       `toml:"cpu_limit,omitempty" json:"cpu_limit" long:"cpu-limit" env:"KUBERNETES_CPU_LIMIT" description:"The CPU allocation given to build containers"`
 	CPULimitOverwriteMaxAllowed                       string                       `toml:"cpu_limit_overwrite_max_allowed,omitempty" json:"cpu_limit_overwrite_max_allowed" long:"cpu-limit-overwrite-max-allowed" env:"KUBERNETES_CPU_LIMIT_OVERWRITE_MAX_ALLOWED" description:"If set, the max amount the cpu limit can be set to. Used with the KUBERNETES_CPU_LIMIT variable in the build."`
@@ -322,6 +328,8 @@ type KubernetesConfig struct {
 	HelperEphemeralStorageLimitOverwriteMaxAllowed    string                       `toml:"helper_ephemeral_storage_limit_overwrite_max_allowed,omitempty" json:"helper_ephemeral_storage_limit_overwrite_max_allowed" long:"helper-ephemeral_storage-limit-overwrite-max-allowed" env:"KUBERNETES_HELPER_EPHEMERAL_STORAGE_LIMIT_OVERWRITE_MAX_ALLOWED" description:"If set, the max amount the helper ephemeral storage limit can be set to. Used with the KUBERNETES_HELPER_EPHEMERAL_STORAGE_LIMIT variable in the build."`
 	HelperEphemeralStorageRequest                     string                       `toml:"helper_ephemeral_storage_request,omitempty" json:"helper_ephemeral_storage_request" long:"helper-ephemeral_storage-request" env:"KUBERNETES_HELPER_EPHEMERAL_STORAGE_REQUEST" description:"The amount of ephemeral storage requested for build helper containers"`
 	HelperEphemeralStorageRequestOverwriteMaxAllowed  string                       `toml:"helper_ephemeral_storage_request_overwrite_max_allowed,omitempty" json:"helper_ephemeral_storage_request_overwrite_max_allowed" long:"helper-ephemeral_storage-request-overwrite-max-allowed" env:"KUBERNETES_HELPER_EPHEMERAL_STORAGE_REQUEST_OVERWRITE_MAX_ALLOWED" description:"If set, the max amount the helper ephemeral storage request can be set to. Used with the KUBERNETES_HELPER_EPHEMERAL_STORAGE_REQUEST variable in the build."`
+	AllowedImages                                     []string                     `toml:"allowed_images,omitempty" json:"allowed_images" long:"allowed-images" env:"KUBERNETES_ALLOWED_IMAGES" description:"Image allowlist"`
+	AllowedServices                                   []string                     `toml:"allowed_services,omitempty" json:"allowed_services" long:"allowed-services" env:"KUBERNETES_ALLOWED_SERVICES" description:"Service allowlist"`
 	PullPolicy                                        StringOrArray                `toml:"pull_policy,omitempty" json:"pull_policy" long:"pull-policy" env:"KUBERNETES_PULL_POLICY" description:"Policy for if/when to pull a container image (never, if-not-present, always). The cluster default will be used if not set"`
 	NodeSelector                                      map[string]string            `toml:"node_selector,omitempty" json:"node_selector" long:"node-selector" env:"KUBERNETES_NODE_SELECTOR" description:"A toml table/json object of key:value. Value is expected to be a string. When set this will create pods on k8s nodes that match all the key:value pairs. Only one selector is supported through environment variable configuration."`
 	NodeTolerations                                   map[string]string            `toml:"node_tolerations,omitempty" json:"node_tolerations" long:"node-tolerations" env:"KUBERNETES_NODE_TOLERATIONS" description:"A toml table/json object of key=value:effect. Value and effect are expected to be strings. When set, pods will tolerate the given taints. Only one toleration is supported through environment variable configuration."`
@@ -345,6 +353,7 @@ type KubernetesConfig struct {
 	CapDrop                                           []string                     `toml:"cap_drop" json:"cap_drop" long:"cap-drop" env:"KUBERNETES_CAP_DROP" description:"Drop Linux capabilities"`
 	DNSPolicy                                         KubernetesDNSPolicy          `toml:"dns_policy,omitempty" json:"dns_policy" long:"dns-policy" env:"KUBERNETES_DNS_POLICY" description:"How Kubernetes should try to resolve DNS from the created pods. If unset, Kubernetes will use the default 'ClusterFirst'. Valid values are: none, default, cluster-first, cluster-first-with-host-net"`
 	DNSConfig                                         KubernetesDNSConfig          `toml:"dns_config" json:"dns_config" description:"Pod DNS config"`
+	ContainerLifecycle                                KubernetesContainerLifecyle  `toml:"container_lifecycle,omitempty" json:"container_lifecycle,omitempty" description:"Actions that the management system should take in response to container lifecycle events"`
 }
 
 //nolint:lll
@@ -433,7 +442,9 @@ type KubernetesPodSecurityContext struct {
 
 //nolint:lll
 type KubernetesAffinity struct {
-	NodeAffinity *KubernetesNodeAffinity `toml:"node_affinity,omitempty" json:"node_affinity" long:"node-affinity" description:"Node affinity is conceptually similar to nodeSelector -- it allows you to constrain which nodes your pod is eligible to be scheduled on, based on labels on the node."`
+	NodeAffinity    *KubernetesNodeAffinity    `toml:"node_affinity,omitempty" json:"node_affinity" long:"node-affinity" description:"Node affinity is conceptually similar to nodeSelector -- it allows you to constrain which nodes your pod is eligible to be scheduled on, based on labels on the node."`
+	PodAffinity     *KubernetesPodAffinity     `toml:"pod_affinity,omitempty" json:"pod_affinity" description:"Pod affinity allows to constrain which nodes your pod is eligible to be scheduled on based on the labels on other pods."`
+	PodAntiAffinity *KubernetesPodAntiAffinity `toml:"pod_anti_affinity,omitempty" json:"pod_anti_affinity" description:"Pod anti-affinity allows to constrain which nodes your pod is eligible to be scheduled on based on the labels on other pods."`
 }
 
 //nolint:lll
@@ -442,12 +453,101 @@ type KubernetesNodeAffinity struct {
 	PreferredDuringSchedulingIgnoredDuringExecution []PreferredSchedulingTerm `toml:"preferred_during_scheduling_ignored_during_execution,omitempty" json:"preferred_during_scheduling_ignored_during_execution"`
 }
 
+//nolint:lll
+type KubernetesPodAffinity struct {
+	RequiredDuringSchedulingIgnoredDuringExecution  []PodAffinityTerm         `toml:"required_during_scheduling_ignored_during_execution,omitempty" json:"required_during_scheduling_ignored_during_execution"`
+	PreferredDuringSchedulingIgnoredDuringExecution []WeightedPodAffinityTerm `toml:"preferred_during_scheduling_ignored_during_execution,omitempty" json:"preferred_during_scheduling_ignored_during_execution"`
+}
+
+//nolint:lll
+type KubernetesPodAntiAffinity struct {
+	RequiredDuringSchedulingIgnoredDuringExecution  []PodAffinityTerm         `toml:"required_during_scheduling_ignored_during_execution,omitempty" json:"required_during_scheduling_ignored_during_execution"`
+	PreferredDuringSchedulingIgnoredDuringExecution []WeightedPodAffinityTerm `toml:"preferred_during_scheduling_ignored_during_execution,omitempty" json:"preferred_during_scheduling_ignored_during_execution"`
+}
+
 //nolint:lll
 type KubernetesHostAliases struct {
 	IP        string   `toml:"ip" json:"ip" long:"ip" description:"The IP address you want to attach hosts to"`
 	Hostnames []string `toml:"hostnames" json:"hostnames" long:"hostnames" description:"A list of hostnames that will be attached to the IP"`
 }
 
+//nolint:lll
+// https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.21/#lifecycle-v1-core
+type KubernetesContainerLifecyle struct {
+	PostStart *KubernetesLifecycleHandler `toml:"post_start,omitempty" json:"post_start,omitempty" description:"PostStart is called immediately after a container is created. If the handler fails, the container is terminated and restarted according to its restart policy. Other management of the container blocks until the hook completes"`
+	PreStop   *KubernetesLifecycleHandler `toml:"pre_stop,omitempty" json:"pre_stop,omitempty" description:"PreStop is called immediately before a container is terminated due to an API request or management event such as liveness/startup probe failure, preemption, resource contention, etc. The handler is not called if the container crashes or exits. The reason for termination is passed to the handler. The Pod's termination grace period countdown begins before the PreStop hooked is executed. Regardless of the outcome of the handler, the container will eventually terminate within the Pod's termination grace period. Other management of the container blocks until the hook completes or until the termination grace period is reached"`
+}
+
+//nolint:lll
+type KubernetesLifecycleHandler struct {
+	Exec      *KubernetesLifecycleExecAction `toml:"exec"  json:"exec" description:"Exec specifies the action to take"`
+	HTTPGet   *KubernetesLifecycleHTTPGet    `toml:"http_get"  json:"http_get" description:"HTTPGet specifies the http request to perform."`
+	TCPSocket *KubernetesLifecycleTCPSocket  `toml:"tcp_socket"  json:"tcp_socket" description:"TCPSocket specifies an action involving a TCP port"`
+}
+
+//nolint:lll
+type KubernetesLifecycleExecAction struct {
+	Command []string `toml:"command" json:"command" description:"Command is the command line to execute inside the container, the working directory for the command  is root ('/') in the container's filesystem. The command is simply exec'd, it is not run inside a shell, so traditional shell instructions ('|', etc) won't work. To use a shell, you need to explicitly call out to that shell. Exit status of 0 is treated as live/healthy and non-zero is unhealthy"`
+}
+
+//nolint:lll
+type KubernetesLifecycleHTTPGet struct {
+	Host        string                             `toml:"host" json:"host" description:"Host name to connect to, defaults to the pod IP. You probably want to set \"Host\" in httpHeaders instead"`
+	HTTPHeaders []KubernetesLifecycleHTTPGetHeader `toml:"http_headers" json:"http_headers" description:"Custom headers to set in the request. HTTP allows repeated headers"`
+	Path        string                             `toml:"path" json:"path" description:"Path to access on the HTTP server"`
+	Port        int                                `toml:"port" json:"port" description:"Number of the port to access on the container. Number must be in the range 1 to 65535"`
+	Scheme      string                             `toml:"scheme" json:"scheme" description:"Scheme to use for connecting to the host. Defaults to HTTP"`
+}
+
+type KubernetesLifecycleHTTPGetHeader struct {
+	Name  string `toml:"name" json:"name" description:"The header field name"`
+	Value string `toml:"value" json:"value" description:"The header field value"`
+}
+
+//nolint:lll
+type KubernetesLifecycleTCPSocket struct {
+	Host string `toml:"host" json:"host" description:"Host name to connect to, defaults to the pod IP. You probably want to set \"Host\" in httpHeaders instead"`
+	Port int    `toml:"port" json:"port" description:"Number of the port to access on the container. Number must be in the range 1 to 65535"`
+}
+
+// ToKubernetesLifecycleHandler converts our lifecycle structs to the ones from the Kubernetes API.
+// We can't use them directly since they don't suppor toml.
+func (h *KubernetesLifecycleHandler) ToKubernetesLifecycleHandler() *api.Handler {
+	kubeHandler := &api.Handler{}
+
+	if h.Exec != nil {
+		kubeHandler.Exec = &api.ExecAction{
+			Command: h.Exec.Command,
+		}
+	}
+	if h.HTTPGet != nil {
+		httpHeaders := []api.HTTPHeader{}
+
+		for _, e := range h.HTTPGet.HTTPHeaders {
+			httpHeaders = append(httpHeaders, api.HTTPHeader{
+				Name:  e.Name,
+				Value: e.Value,
+			})
+		}
+
+		kubeHandler.HTTPGet = &api.HTTPGetAction{
+			Host:        h.HTTPGet.Host,
+			Port:        intstr.FromInt(h.HTTPGet.Port),
+			Path:        h.HTTPGet.Path,
+			Scheme:      api.URIScheme(h.HTTPGet.Scheme),
+			HTTPHeaders: httpHeaders,
+		}
+	}
+	if h.TCPSocket != nil {
+		kubeHandler.TCPSocket = &api.TCPSocketAction{
+			Host: h.TCPSocket.Host,
+			Port: intstr.FromInt(h.TCPSocket.Port),
+		}
+	}
+
+	return kubeHandler
+}
+
 type NodeSelector struct {
 	NodeSelectorTerms []NodeSelectorTerm `toml:"node_selector_terms" json:"node_selector_terms"`
 }
@@ -457,6 +557,11 @@ type PreferredSchedulingTerm struct {
 	Preference NodeSelectorTerm `toml:"preference" json:"preference"`
 }
 
+type WeightedPodAffinityTerm struct {
+	Weight          int32           `toml:"weight" json:"weight"`
+	PodAffinityTerm PodAffinityTerm `toml:"pod_affinity_term" json:"pod_affinity_term"`
+}
+
 type NodeSelectorTerm struct {
 	MatchExpressions []NodeSelectorRequirement `toml:"match_expressions,omitempty" json:"match_expressions"`
 	MatchFields      []NodeSelectorRequirement `toml:"match_fields,omitempty" json:"match_fields"`
@@ -469,6 +574,18 @@ type NodeSelectorRequirement struct {
 	Values   []string `toml:"values,omitempty" json:"values"`
 }
 
+type PodAffinityTerm struct {
+	LabelSelector     *LabelSelector `toml:"label_selector,omitempty" json:"label_selector"`
+	Namespaces        []string       `toml:"namespaces,omitempty" json:"namespaces"`
+	TopologyKey       string         `toml:"topology_key,omitempty" json:"topology_key"`
+	NamespaceSelector *LabelSelector `toml:"namespace_selector,omitempty" json:"namespace_selector"`
+}
+
+type LabelSelector struct {
+	MatchLabels      map[string]string         `toml:"match_labels,omitempty" json:"match_labels"`
+	MatchExpressions []NodeSelectorRequirement `toml:"match_expressions,omitempty" json:"match_expressions"`
+}
+
 //nolint:lll
 type Service struct {
 	Name       string   `toml:"name" long:"name" description:"The image path for the service"`
@@ -797,6 +914,14 @@ func (c *KubernetesConfig) GetAffinity() *api.Affinity {
 		affinity.NodeAffinity = c.GetNodeAffinity()
 	}
 
+	if c.Affinity.PodAffinity != nil {
+		affinity.PodAffinity = c.GetPodAffinity()
+	}
+
+	if c.Affinity.PodAntiAffinity != nil {
+		affinity.PodAntiAffinity = c.GetPodAntiAffinity()
+	}
+
 	return &affinity
 }
 
@@ -820,20 +945,28 @@ func (c *KubernetesConfig) GetDNSConfig() *api.PodDNSConfig {
 	return &config
 }
 
-//nolint:lll
 func (c *KubernetesConfig) GetNodeAffinity() *api.NodeAffinity {
 	var nodeAffinity api.NodeAffinity
 
 	if c.Affinity.NodeAffinity.RequiredDuringSchedulingIgnoredDuringExecution != nil {
-		nodeAffinity.RequiredDuringSchedulingIgnoredDuringExecution = c.Affinity.NodeAffinity.RequiredDuringSchedulingIgnoredDuringExecution.GetNodeSelector()
+		nodeSelector := c.Affinity.NodeAffinity.RequiredDuringSchedulingIgnoredDuringExecution.GetNodeSelector()
+		nodeAffinity.RequiredDuringSchedulingIgnoredDuringExecution = nodeSelector
 	}
 
 	for _, preferred := range c.Affinity.NodeAffinity.PreferredDuringSchedulingIgnoredDuringExecution {
-		nodeAffinity.PreferredDuringSchedulingIgnoredDuringExecution = append(nodeAffinity.PreferredDuringSchedulingIgnoredDuringExecution, preferred.GetPreferredSchedulingTerm())
+		nodeAffinity.PreferredDuringSchedulingIgnoredDuringExecution = append(
+			nodeAffinity.PreferredDuringSchedulingIgnoredDuringExecution,
+			preferred.GetPreferredSchedulingTerm(),
+		)
 	}
 	return &nodeAffinity
 }
 
+// GetContainerLifecycle returns the container lifecycle configuration
+func (c *KubernetesConfig) GetContainerLifecycle() KubernetesContainerLifecyle {
+	return c.ContainerLifecycle
+}
+
 func (c *NodeSelector) GetNodeSelector() *api.NodeSelector {
 	var nodeSelector api.NodeSelector
 	for _, selector := range c.NodeSelectorTerms {
@@ -850,15 +983,92 @@ func (c *NodeSelectorRequirement) GetNodeSelectorRequirement() api.NodeSelectorR
 	}
 }
 
-//nolint:lll
+func (c *LabelSelector) GetLabelSelectorMatchExpressions() []metav1.LabelSelectorRequirement {
+	var labelSelectorRequirement []metav1.LabelSelectorRequirement
+
+	for _, label := range c.MatchExpressions {
+		var expression = metav1.LabelSelectorRequirement{
+			Key:      label.Key,
+			Operator: metav1.LabelSelectorOperator(label.Operator),
+			Values:   label.Values,
+		}
+		labelSelectorRequirement = append(labelSelectorRequirement, expression)
+	}
+
+	return labelSelectorRequirement
+}
+
+func (c *KubernetesConfig) GetPodAffinity() *api.PodAffinity {
+	var podAffinity api.PodAffinity
+
+	for _, required := range c.Affinity.PodAffinity.RequiredDuringSchedulingIgnoredDuringExecution {
+		podAffinity.RequiredDuringSchedulingIgnoredDuringExecution = append(
+			podAffinity.RequiredDuringSchedulingIgnoredDuringExecution,
+			required.GetPodAffinityTerm(),
+		)
+	}
+
+	for _, preferred := range c.Affinity.PodAffinity.PreferredDuringSchedulingIgnoredDuringExecution {
+		podAffinity.PreferredDuringSchedulingIgnoredDuringExecution = append(
+			podAffinity.PreferredDuringSchedulingIgnoredDuringExecution,
+			preferred.GetWeightedPodAffinityTerm(),
+		)
+	}
+
+	return &podAffinity
+}
+
+func (c *KubernetesConfig) GetPodAntiAffinity() *api.PodAntiAffinity {
+	var podAntiAffinity api.PodAntiAffinity
+
+	for _, required := range c.Affinity.PodAntiAffinity.RequiredDuringSchedulingIgnoredDuringExecution {
+		podAntiAffinity.RequiredDuringSchedulingIgnoredDuringExecution = append(
+			podAntiAffinity.RequiredDuringSchedulingIgnoredDuringExecution,
+			required.GetPodAffinityTerm(),
+		)
+	}
+
+	for _, preferred := range c.Affinity.PodAntiAffinity.PreferredDuringSchedulingIgnoredDuringExecution {
+		podAntiAffinity.PreferredDuringSchedulingIgnoredDuringExecution = append(
+			podAntiAffinity.PreferredDuringSchedulingIgnoredDuringExecution,
+			preferred.GetWeightedPodAffinityTerm(),
+		)
+	}
+
+	return &podAntiAffinity
+}
+
+func (c *PodAffinityTerm) GetPodAffinityTerm() api.PodAffinityTerm {
+	return api.PodAffinityTerm{
+		LabelSelector:     c.GetLabelSelector(),
+		Namespaces:        c.Namespaces,
+		TopologyKey:       c.TopologyKey,
+		NamespaceSelector: c.GetNamespaceSelector(),
+	}
+}
+
+func (c *WeightedPodAffinityTerm) GetWeightedPodAffinityTerm() api.WeightedPodAffinityTerm {
+	return api.WeightedPodAffinityTerm{
+		Weight:          c.Weight,
+		PodAffinityTerm: c.PodAffinityTerm.GetPodAffinityTerm(),
+	}
+}
+
 func (c *NodeSelectorTerm) GetNodeSelectorTerm() api.NodeSelectorTerm {
 	var nodeSelectorTerm = api.NodeSelectorTerm{}
 	for _, expression := range c.MatchExpressions {
-		nodeSelectorTerm.MatchExpressions = append(nodeSelectorTerm.MatchExpressions, expression.GetNodeSelectorRequirement())
+		nodeSelectorTerm.MatchExpressions = append(
+			nodeSelectorTerm.MatchExpressions,
+			expression.GetNodeSelectorRequirement(),
+		)
 	}
 	for _, fields := range c.MatchFields {
-		nodeSelectorTerm.MatchFields = append(nodeSelectorTerm.MatchFields, fields.GetNodeSelectorRequirement())
+		nodeSelectorTerm.MatchFields = append(
+			nodeSelectorTerm.MatchFields,
+			fields.GetNodeSelectorRequirement(),
+		)
 	}
+
 	return nodeSelectorTerm
 }
 
@@ -869,6 +1079,28 @@ func (c *PreferredSchedulingTerm) GetPreferredSchedulingTerm() api.PreferredSche
 	}
 }
 
+func (c *PodAffinityTerm) GetLabelSelector() *metav1.LabelSelector {
+	if c.LabelSelector == nil {
+		return nil
+	}
+
+	return &metav1.LabelSelector{
+		MatchLabels:      c.LabelSelector.MatchLabels,
+		MatchExpressions: c.LabelSelector.GetLabelSelectorMatchExpressions(),
+	}
+}
+
+func (c *PodAffinityTerm) GetNamespaceSelector() *metav1.LabelSelector {
+	if c.NamespaceSelector == nil {
+		return nil
+	}
+
+	return &metav1.LabelSelector{
+		MatchLabels:      c.NamespaceSelector.MatchLabels,
+		MatchExpressions: c.NamespaceSelector.GetLabelSelectorMatchExpressions(),
+	}
+}
+
 func (c *KubernetesConfig) GetHostAliases() []api.HostAlias {
 	var hostAliases []api.HostAlias
 
@@ -1033,6 +1265,22 @@ func (c *RunnerConfig) DeepCopy() (*RunnerConfig, error) {
 	return &r, err
 }
 
+func (c *RunnerConfig) logWarnings() {
+	if c.SSH != nil {
+		if c.SSH.DisableStrictHostKeyChecking == nil {
+			// TODO: Remove in 15.0 https://gitlab.com/gitlab-org/gitlab-runner/-/issues/28192
+			logrus.Warningln("[[ssh.disable_strict_host_key_checking]] is not set. " +
+				"Starting from 15.0, strict host checking will be enabled unless explicitly disabled.")
+		}
+
+		if c.SSH.ShouldDisableStrictHostKeyChecking() {
+			logrus.Warningln("SSH strict host checking is disabled. Consider setting " +
+				"[[ssh.disable_strict_host_key_checking]] " +
+				"to false (https://docs.gitlab.com/runner/executors/ssh.html#security).")
+		}
+	}
+}
+
 func NewConfig() *Config {
 	return &Config{
 		Concurrent: 1,
@@ -1065,6 +1313,8 @@ func (c *Config) LoadConfig(configFile string) error {
 	}
 
 	for _, runner := range c.Runners {
+		runner.logWarnings()
+
 		if runner.Machine == nil {
 			continue
 		}
