@@ -111,6 +111,11 @@ func TestCacheS3Config_ShouldUseIAMCredentials(t *testing.T) {
 }
 
 func TestConfigParse(t *testing.T) {
+	httpHeaders := []KubernetesLifecycleHTTPGetHeader{
+		{Name: "header_name_1", Value: "header_value_1"},
+		{Name: "header_name_2", Value: "header_value_2"},
+	}
+
 	tests := map[string]struct {
 		config         string
 		validateConfig func(t *testing.T, config *Config)
@@ -297,6 +302,173 @@ func TestConfigParse(t *testing.T) {
 				assert.Equal(t, []string{"e2e-az1"}, nodeAffinity.RequiredDuringSchedulingIgnoredDuringExecution.NodeSelectorTerms[1].MatchFields[0].Values)
 			},
 		},
+
+		//nolint:lll
+		"check pod affinities": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						[runners.kubernetes.affinity]
+							[runners.kubernetes.affinity.pod_affinity]
+								[[runners.kubernetes.affinity.pod_affinity.required_during_scheduling_ignored_during_execution]]
+									topology_key = "failure-domain.beta.kubernetes.io/zone"
+									namespaces = ["namespace_1", "namespace_2"]
+									[runners.kubernetes.affinity.pod_affinity.required_during_scheduling_ignored_during_execution.label_selector]
+										[[runners.kubernetes.affinity.pod_affinity.required_during_scheduling_ignored_during_execution.label_selector.match_expressions]]
+											key = "security"
+											operator = "In"
+											values = ["S1"]
+									[runners.kubernetes.affinity.pod_affinity.required_during_scheduling_ignored_during_execution.namespace_selector]
+										[[runners.kubernetes.affinity.pod_affinity.required_during_scheduling_ignored_during_execution.namespace_selector.match_expressions]]
+											key = "security"
+											operator = "In"
+											values = ["S1"]
+								
+								[[runners.kubernetes.affinity.pod_affinity.preferred_during_scheduling_ignored_during_execution]]
+								weight = 100
+								[runners.kubernetes.affinity.pod_affinity.preferred_during_scheduling_ignored_during_execution.pod_affinity_term]
+									topology_key = "failure-domain.beta.kubernetes.io/zone"
+									[runners.kubernetes.affinity.pod_affinity.preferred_during_scheduling_ignored_during_execution.pod_affinity_term.label_selector]
+										[[runners.kubernetes.affinity.pod_affinity.preferred_during_scheduling_ignored_during_execution.pod_affinity_term.label_selector.match_expressions]]
+											key = "security_2"
+											operator = "In"
+											values = ["S2"]
+									[runners.kubernetes.affinity.pod_affinity.preferred_during_scheduling_ignored_during_execution.pod_affinity_term.namespace_selector]
+										[[runners.kubernetes.affinity.pod_affinity.preferred_during_scheduling_ignored_during_execution.pod_affinity_term.namespace_selector.match_expressions]]
+											key = "security_2"
+											operator = "In"
+											values = ["S2"]
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+				require.NotNil(t, config.Runners[0].Kubernetes.Affinity)
+				require.NotNil(t, config.Runners[0].Kubernetes.Affinity.PodAffinity)
+
+				podAffinity := config.Runners[0].Kubernetes.Affinity.PodAffinity
+				require.Len(t, podAffinity.RequiredDuringSchedulingIgnoredDuringExecution, 1)
+				required := podAffinity.RequiredDuringSchedulingIgnoredDuringExecution
+
+				assert.Equal(t, "failure-domain.beta.kubernetes.io/zone", required[0].TopologyKey)
+				assert.Equal(t, []string{"namespace_1", "namespace_2"}, required[0].Namespaces)
+
+				require.NotNil(t, required[0].LabelSelector)
+				require.Len(t, required[0].LabelSelector.MatchExpressions, 1)
+				requiredMatchExp := required[0].LabelSelector.MatchExpressions[0]
+				assert.Equal(t, "security", requiredMatchExp.Key)
+				assert.Equal(t, "In", requiredMatchExp.Operator)
+				assert.Equal(t, []string{"S1"}, requiredMatchExp.Values)
+
+				require.NotNil(t, required[0].NamespaceSelector)
+				require.Len(t, required[0].NamespaceSelector.MatchExpressions, 1)
+				requiredMatchExp = required[0].NamespaceSelector.MatchExpressions[0]
+				assert.Equal(t, "security", requiredMatchExp.Key)
+				assert.Equal(t, "In", requiredMatchExp.Operator)
+				assert.Equal(t, []string{"S1"}, requiredMatchExp.Values)
+
+				require.Len(t, podAffinity.PreferredDuringSchedulingIgnoredDuringExecution, 1)
+				preferred := podAffinity.PreferredDuringSchedulingIgnoredDuringExecution
+				assert.Equal(t, int32(100), preferred[0].Weight)
+				assert.Empty(t, preferred[0].PodAffinityTerm.Namespaces)
+				assert.Equal(t, "failure-domain.beta.kubernetes.io/zone", preferred[0].PodAffinityTerm.TopologyKey)
+
+				require.NotNil(t, preferred[0].PodAffinityTerm.LabelSelector)
+				require.Len(t, preferred[0].PodAffinityTerm.LabelSelector.MatchExpressions, 1)
+				preferredMatchExp := preferred[0].PodAffinityTerm.LabelSelector.MatchExpressions[0]
+				assert.Equal(t, "security_2", preferredMatchExp.Key)
+				assert.Equal(t, "In", preferredMatchExp.Operator)
+				assert.Equal(t, []string{"S2"}, preferredMatchExp.Values)
+
+				require.NotNil(t, preferred[0].PodAffinityTerm.NamespaceSelector)
+				require.Len(t, preferred[0].PodAffinityTerm.NamespaceSelector.MatchExpressions, 1)
+				preferredMatchExp = preferred[0].PodAffinityTerm.NamespaceSelector.MatchExpressions[0]
+				assert.Equal(t, "security_2", preferredMatchExp.Key)
+				assert.Equal(t, "In", preferredMatchExp.Operator)
+				assert.Equal(t, []string{"S2"}, preferredMatchExp.Values)
+			},
+		},
+		//nolint:lll
+		"check pod anti affinities": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						[runners.kubernetes.affinity]
+							[runners.kubernetes.affinity.pod_anti_affinity]
+								[[runners.kubernetes.affinity.pod_anti_affinity.required_during_scheduling_ignored_during_execution]]
+									topology_key = "failure-domain.beta.kubernetes.io/zone"
+									namespaces = ["namespace_1", "namespace_2"]
+									[runners.kubernetes.affinity.pod_anti_affinity.required_during_scheduling_ignored_during_execution.label_selector]
+										[[runners.kubernetes.affinity.pod_anti_affinity.required_during_scheduling_ignored_during_execution.label_selector.match_expressions]]
+											key = "security"
+											operator = "In"
+											values = ["S1"]
+									[runners.kubernetes.affinity.pod_anti_affinity.required_during_scheduling_ignored_during_execution.namespace_selector]
+										[[runners.kubernetes.affinity.pod_anti_affinity.required_during_scheduling_ignored_during_execution.namespace_selector.match_expressions]]
+											key = "security"
+											operator = "In"
+											values = ["S1"]
+								
+								[[runners.kubernetes.affinity.pod_anti_affinity.preferred_during_scheduling_ignored_during_execution]]
+								weight = 100
+								[runners.kubernetes.affinity.pod_anti_affinity.preferred_during_scheduling_ignored_during_execution.pod_affinity_term]
+									topology_key = "failure-domain.beta.kubernetes.io/zone"
+									[runners.kubernetes.affinity.pod_anti_affinity.preferred_during_scheduling_ignored_during_execution.pod_affinity_term.label_selector]
+										[[runners.kubernetes.affinity.pod_anti_affinity.preferred_during_scheduling_ignored_during_execution.pod_affinity_term.label_selector.match_expressions]]
+											key = "security_2"
+											operator = "In"
+											values = ["S2"]
+									[runners.kubernetes.affinity.pod_anti_affinity.preferred_during_scheduling_ignored_during_execution.pod_affinity_term.namespace_selector]
+										[[runners.kubernetes.affinity.pod_anti_affinity.preferred_during_scheduling_ignored_during_execution.pod_affinity_term.namespace_selector.match_expressions]]
+											key = "security_2"
+											operator = "In"
+											values = ["S2"]
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+				require.NotNil(t, config.Runners[0].Kubernetes.Affinity)
+				require.NotNil(t, config.Runners[0].Kubernetes.Affinity.PodAntiAffinity)
+
+				podAntiAffinity := config.Runners[0].Kubernetes.Affinity.PodAntiAffinity
+				require.Len(t, podAntiAffinity.RequiredDuringSchedulingIgnoredDuringExecution, 1)
+				required := podAntiAffinity.RequiredDuringSchedulingIgnoredDuringExecution[0]
+
+				assert.Equal(t, "failure-domain.beta.kubernetes.io/zone", required.TopologyKey)
+				assert.Equal(t, []string{"namespace_1", "namespace_2"}, required.Namespaces)
+
+				require.NotNil(t, required.LabelSelector)
+				require.Len(t, required.LabelSelector.MatchExpressions, 1)
+				requiredMatchExp := required.LabelSelector.MatchExpressions[0]
+				assert.Equal(t, "security", requiredMatchExp.Key)
+				assert.Equal(t, "In", requiredMatchExp.Operator)
+				assert.Equal(t, []string{"S1"}, requiredMatchExp.Values)
+
+				require.NotNil(t, required.NamespaceSelector)
+				require.Len(t, required.NamespaceSelector.MatchExpressions, 1)
+				requiredMatchExp = required.NamespaceSelector.MatchExpressions[0]
+				assert.Equal(t, "security", requiredMatchExp.Key)
+				assert.Equal(t, "In", requiredMatchExp.Operator)
+				assert.Equal(t, []string{"S1"}, requiredMatchExp.Values)
+
+				require.Len(t, podAntiAffinity.PreferredDuringSchedulingIgnoredDuringExecution, 1)
+				preferred := podAntiAffinity.PreferredDuringSchedulingIgnoredDuringExecution[0]
+				assert.Equal(t, int32(100), preferred.Weight)
+				assert.Empty(t, preferred.PodAffinityTerm.Namespaces)
+				assert.Equal(t, "failure-domain.beta.kubernetes.io/zone", preferred.PodAffinityTerm.TopologyKey)
+
+				require.NotNil(t, preferred.PodAffinityTerm.LabelSelector)
+				require.Len(t, preferred.PodAffinityTerm.LabelSelector.MatchExpressions, 1)
+				preferredMatchExp := preferred.PodAffinityTerm.LabelSelector.MatchExpressions[0]
+				assert.Equal(t, "security_2", preferredMatchExp.Key)
+				assert.Equal(t, "In", preferredMatchExp.Operator)
+				assert.Equal(t, []string{"S2"}, preferredMatchExp.Values)
+
+				require.NotNil(t, preferred.PodAffinityTerm.NamespaceSelector)
+				require.Len(t, preferred.PodAffinityTerm.NamespaceSelector.MatchExpressions, 1)
+				preferredMatchExp = preferred.PodAffinityTerm.NamespaceSelector.MatchExpressions[0]
+				assert.Equal(t, "security_2", preferredMatchExp.Key)
+				assert.Equal(t, "In", preferredMatchExp.Operator)
+				assert.Equal(t, []string{"S2"}, preferredMatchExp.Values)
+			},
+		},
 		"check that GracefulKillTimeout and ForceKillTimeout can't be set": {
 			config: `
 				[[runners]]
@@ -392,6 +564,152 @@ func TestConfigParse(t *testing.T) {
 				assert.Equal(t, api.DNSClusterFirst, dnsPolicy)
 			},
 		},
+		"check empty container lifecycle": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						namespace = "default"
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				lifecycleCfg := config.Runners[0].Kubernetes.GetContainerLifecycle()
+				assert.Nil(t, lifecycleCfg.PostStart)
+				assert.Nil(t, lifecycleCfg.PreStop)
+			},
+		},
+		"check postStart execAction configuration": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						namespace = "default"
+						[runners.kubernetes.container_lifecycle.post_start.exec]
+							command = ["ls", "-l"]
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				lifecycleCfg := config.Runners[0].Kubernetes.GetContainerLifecycle()
+				assert.NotNil(t, lifecycleCfg.PostStart)
+
+				assert.Equal(t, []string{"ls", "-l"}, lifecycleCfg.PostStart.Exec.Command)
+				assert.Nil(t, nil, lifecycleCfg.PostStart.HTTPGet)
+				assert.Nil(t, nil, lifecycleCfg.PostStart.TCPSocket)
+			},
+		},
+		"check postStart httpGetAction configuration": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						namespace = "default"
+						[runners.kubernetes.container_lifecycle.post_start.http_get]
+							port = 8080
+							host = "localhost"
+							path = "/test"
+							[[runners.kubernetes.container_lifecycle.post_start.http_get.http_headers]]
+								name = "header_name_1"
+								value = "header_value_1"
+							[[runners.kubernetes.container_lifecycle.post_start.http_get.http_headers]]
+								name = "header_name_2"
+								value = "header_value_2"
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				lifecycleCfg := config.Runners[0].Kubernetes.GetContainerLifecycle()
+				assert.NotNil(t, lifecycleCfg.PostStart)
+
+				assert.Equal(t, 8080, lifecycleCfg.PostStart.HTTPGet.Port)
+				assert.Equal(t, "localhost", lifecycleCfg.PostStart.HTTPGet.Host)
+				assert.Equal(t, "/test", lifecycleCfg.PostStart.HTTPGet.Path)
+				assert.Equal(t, httpHeaders, lifecycleCfg.PostStart.HTTPGet.HTTPHeaders)
+			},
+		},
+		"check postStart tcpSocketAction configuration": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						namespace = "default"
+						[runners.kubernetes.container_lifecycle.post_start.tcp_socket]
+							port = 8080
+							host = "localhost"
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				lifecycleCfg := config.Runners[0].Kubernetes.GetContainerLifecycle()
+				assert.NotNil(t, lifecycleCfg.PostStart)
+
+				assert.Equal(t, 8080, lifecycleCfg.PostStart.TCPSocket.Port)
+				assert.Equal(t, "localhost", lifecycleCfg.PostStart.TCPSocket.Host)
+			},
+		},
+		"check preStop execAction configuration": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						namespace = "default"
+						[runners.kubernetes.container_lifecycle.pre_stop.exec]
+							command = ["ls", "-l"]
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				lifecycleCfg := config.Runners[0].Kubernetes.GetContainerLifecycle()
+				assert.NotNil(t, lifecycleCfg.PreStop)
+
+				assert.Equal(t, []string{"ls", "-l"}, lifecycleCfg.PreStop.Exec.Command)
+				assert.Nil(t, nil, lifecycleCfg.PreStop.HTTPGet)
+				assert.Nil(t, nil, lifecycleCfg.PreStop.TCPSocket)
+			},
+		},
+		"check preStop httpGetAction configuration": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						namespace = "default"
+						[runners.kubernetes.container_lifecycle.pre_stop.http_get]
+						port = 8080
+						host = "localhost"
+						path = "/test"
+						[[runners.kubernetes.container_lifecycle.pre_stop.http_get.http_headers]]
+							name = "header_name_1"
+							value = "header_value_1"
+						[[runners.kubernetes.container_lifecycle.pre_stop.http_get.http_headers]]
+							name = "header_name_2"
+							value = "header_value_2"
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				lifecycleCfg := config.Runners[0].Kubernetes.GetContainerLifecycle()
+				assert.NotNil(t, lifecycleCfg.PreStop)
+
+				assert.Equal(t, 8080, lifecycleCfg.PreStop.HTTPGet.Port)
+				assert.Equal(t, "localhost", lifecycleCfg.PreStop.HTTPGet.Host)
+				assert.Equal(t, "/test", lifecycleCfg.PreStop.HTTPGet.Path)
+				assert.Equal(t, httpHeaders, lifecycleCfg.PreStop.HTTPGet.HTTPHeaders)
+			},
+		},
+		"check preStop tcpSocketAction configuration": {
+			config: `
+				[[runners]]
+					[runners.kubernetes]
+						namespace = "default"
+						[runners.kubernetes.container_lifecycle.pre_stop.tcp_socket]
+							port = 8080
+							host = "localhost"
+			`,
+			validateConfig: func(t *testing.T, config *Config) {
+				require.Len(t, config.Runners, 1)
+
+				lifecycleCfg := config.Runners[0].Kubernetes.GetContainerLifecycle()
+				assert.NotNil(t, lifecycleCfg.PreStop)
+
+				assert.Equal(t, 8080, lifecycleCfg.PreStop.TCPSocket.Port)
+				assert.Equal(t, "localhost", lifecycleCfg.PreStop.TCPSocket.Host)
+			},
+		},
 	}
 
 	for tn, tt := range tests {
