@@ -368,6 +368,8 @@ type Secrets map[string]Secret
 
 type Secret struct {
 	Vault *VaultSecret `json:"vault,omitempty"`
+
+	File *bool `json:"file,omitempty"`
 }
 
 type VaultSecret struct {
@@ -407,6 +409,18 @@ func (s Secret) expandVariables(vars JobVariables) {
 	}
 }
 
+// IsFile defines whether the variable should be of type FILE or no.
+//
+// The default behavior is to represent the variable as FILE type.
+// If defined by the user - set to whatever was chosen.
+func (s Secret) IsFile() bool {
+	if s.File == nil {
+		return SecretVariableDefaultsToFile
+	}
+
+	return *s.File
+}
+
 func (s *VaultSecret) expandVariables(vars JobVariables) {
 	s.Server.expandVariables(vars)
 	s.Engine.expandVariables(vars)
