@@ -20,6 +20,8 @@ import (
 	"testing"
 	"time"
 
+	"gitlab.com/gitlab-org/gitlab-runner/shells/shellstest"
+
 	"github.com/docker/docker/api/types"
 	"github.com/docker/docker/api/types/filters"
 	"github.com/hashicorp/go-version"
@@ -266,6 +268,23 @@ func TestDockerCommandSuccessRunFileVariableContent(t *testing.T) {
 	assert.Contains(t, out, fmt.Sprintf("%X", sha1.Sum([]byte(value))))
 }
 
+func TestBuildScriptSections(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		if shell == "cmd" || shell == "pwsh" || shell == "powershell" {
+			// support for pwsh and powershell tracked in https://gitlab.com/gitlab-org/gitlab-runner/-/issues/28119
+			t.Skip("CMD, pwsh, powershell not supported")
+		}
+
+		build := getBuildForOS(t, func() (common.JobResponse, error) {
+			return common.GetRemoteSuccessfulBuild()
+		})
+
+		build.Runner.RunnerSettings.Shell = shell
+
+		buildtest.RunBuildWithSections(t, &build)
+	})
+}
+
 func TestDockerCommandUsingCustomClonePath(t *testing.T) {
 	helpers.SkipIntegrationTests(t, "docker", "info")
 
@@ -513,6 +532,23 @@ func TestDockerCommandMissingServiceImage(t *testing.T) {
 	assert.Contains(t, err.Error(), contains)
 }
 
+// TestDockerCommandPullingImageRequestTimeout tests if the request timeout cancelation
+// is categorized as a script failure. It also helps to monitor the eventual error message change in
+// future docker library versions
+func TestDockerCommandPullingImageRequestTimeout(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "docker", "info")
+
+	build := getBuildForOS(t, common.GetSuccessfulBuild)
+	build.Runner.RunnerSettings.Docker.Image = "docker.repo.bogus.com.au/docker:18.09.7-dind"
+
+	var buildError *common.BuildError
+	err := build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
+	require.ErrorAs(t, err, &buildError)
+
+	assert.True(t, docker.IsSystemError(err), "expected docker system error is missing in error chain")
+	assert.Contains(t, err.Error(), "request canceled while waiting for connection")
+}
+
 func TestDockerCommandBuildCancel(t *testing.T) {
 	helpers.SkipIntegrationTests(t, "docker", "info")
 
@@ -1603,3 +1639,64 @@ func TestBuildLogLimitExceeded(t *testing.T) {
 
 	buildtest.RunRemoteBuildWithJobOutputLimitExceeded(t, getRunnerConfigForOS(t), nil)
 }
+
+func TestCleanupProjectGitClone(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "docker", "info")
+
+	build := getBuildForOS(t, common.GetRemoteSuccessfulBuild)
+	buildtest.RunBuildWithCleanupGitClone(t, &build)
+}
+
+func TestCleanupProjectGitFetch(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "docker", "info")
+
+	untrackedFilename := "untracked"
+
+	build := getBuildForOS(t, func() (common.JobResponse, error) {
+		return common.GetRemoteBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(untrackedFilename, "", "")...,
+		)
+	})
+
+	buildtest.RunBuildWithCleanupGitFetch(t, &build, untrackedFilename)
+}
+
+func TestCleanupProjectGitSubmoduleNormal(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "docker", "info")
+
+	untrackedFile := "untracked"
+	untrackedSubmoduleFile := "untracked_submodule"
+
+	build := getBuildForOS(t, func() (common.JobResponse, error) {
+		return common.GetRemoteBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(untrackedFile, untrackedSubmoduleFile, "")...,
+		)
+	})
+
+	buildtest.RunBuildWithCleanupNormalSubmoduleStrategy(t, &build, untrackedFile, untrackedSubmoduleFile)
+}
+
+func TestCleanupProjectGitSubmoduleRecursive(t *testing.T) {
+	helpers.SkipIntegrationTests(t, "docker", "info")
+
+	untrackedFile := "untracked"
+	untrackedSubmoduleFile := "untracked_submodule"
+	untrackedSubSubmoduleFile := "untracked_submodule_submodule"
+
+	build := getBuildForOS(t, func() (common.JobResponse, error) {
+		return common.GetRemoteBuildResponse(
+			buildtest.GetNewUntrackedFileIntoSubmodulesCommands(
+				untrackedFile,
+				untrackedSubmoduleFile,
+				untrackedSubSubmoduleFile)...,
+		)
+	})
+
+	buildtest.RunBuildWithCleanupRecursiveSubmoduleStrategy(
+		t,
+		&build,
+		untrackedFile,
+		untrackedSubmoduleFile,
+		untrackedSubSubmoduleFile,
+	)
+}
