@@ -1,6 +1,9 @@
+// +build !integration
+
 package volumes
 
 import (
+	"path/filepath"
 	"testing"
 
 	"github.com/stretchr/testify/assert"
@@ -74,7 +77,11 @@ func TestManagedList_Add(t *testing.T) {
 		},
 		"add duplicated path": {
 			path:          "/duplicate",
-			expectedError: NewErrVolumeAlreadyDefined("/duplicate"),
+			expectedError: NewErrVolumeAlreadyDefined(filepath.FromSlash("/duplicate")),
+		},
+		"add non-normalized duplicated path": {
+			path:          "/duplicate/",
+			expectedError: NewErrVolumeAlreadyDefined(filepath.FromSlash("/duplicate")),
 		},
 		"add child path": {
 			path: "/duplicate/child",
