@@ -11,7 +11,6 @@ import (
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/docker"
-	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 )
 
 type machineProvider struct {
@@ -30,6 +29,8 @@ type machineProvider struct {
 	totalActions      *prometheus.CounterVec
 	currentStatesDesc *prometheus.Desc
 	creationHistogram prometheus.Histogram
+	stoppingHistogram prometheus.Histogram
+	removalHistogram  prometheus.Histogram
 }
 
 func (m *machineProvider) machineDetails(name string, acquire bool) *machineDetails {
@@ -116,12 +117,7 @@ func (m *machineProvider) createWithGrowthCapacity(
 	logger := logrus.WithField("name", details.Name)
 	started := time.Now()
 
-	err := m.reprovisionMachineOnCreationFailure(
-		logger,
-		config,
-		details,
-		m.machine.Create(config.Machine.MachineDriver, details.Name, config.Machine.MachineOptions...),
-	)
+	err := m.machine.Create(config.Machine.MachineDriver, details.Name, config.Machine.MachineOptions...)
 	if err != nil {
 		logger.WithField("time", time.Since(started)).
 			WithError(err).
@@ -149,29 +145,6 @@ func (m *machineProvider) createWithGrowthCapacity(
 	errCh <- err
 }
 
-func (m *machineProvider) reprovisionMachineOnCreationFailure(
-	logger logrus.FieldLogger,
-	config *common.RunnerConfig,
-	details *machineDetails,
-	err error,
-) error {
-	skipProvision := config.IsFeatureFlagOn(featureflags.SkipDockerMachineProvisionOnCreationFailure)
-	if skipProvision {
-		logger.WithError(err).Infof("Skipping provision retry on failed machine")
-		return err
-	}
-
-	for i := 0; i < 3 && err != nil; i++ {
-		details.RetryCount++
-		logger.WithError(err).
-			Warningln("Machine creation failed, trying to provision")
-		time.Sleep(provisionRetryInterval)
-		err = m.machine.Provision(details.Name)
-	}
-
-	return err
-}
-
 func (m *machineProvider) findFreeMachine(skipCache bool, machines ...string) (details *machineDetails) {
 	// Enumerate all machines in reverse order, to always take the newest machines first
 	for idx := range machines {
@@ -294,18 +267,20 @@ func (m *machineProvider) removeMachine(details *machineDetails) (err error) {
 		defer m.stuckRemoveLock.Unlock()
 	}
 
-	details.logger().
-		Warningln("Stopping machine")
-	err = m.machine.Stop(details.Name, machineStopCommandTimeout)
+	details.logger().Warningln("Stopping machine")
+	err = runHistogramCountedOperation(m.stoppingHistogram, func() error {
+		return m.machine.Stop(details.Name, machineStopCommandTimeout)
+	})
 	if err != nil {
 		details.logger().
 			WithError(err).
 			Warningln("Error while stopping machine")
 	}
 
-	details.logger().
-		Warningln("Removing machine")
-	err = m.machine.Remove(details.Name)
+	details.logger().Warningln("Removing machine")
+	err = runHistogramCountedOperation(m.removalHistogram, func() error {
+		return m.machine.Remove(details.Name)
+	})
 	if err != nil {
 		details.RetryCount++
 		time.Sleep(removeRetryInterval)
@@ -315,6 +290,14 @@ func (m *machineProvider) removeMachine(details *machineDetails) (err error) {
 	return nil
 }
 
+func runHistogramCountedOperation(histogram prometheus.Histogram, operation func() error) error {
+	startedAt := time.Now()
+	err := operation()
+	histogram.Observe(time.Since(startedAt).Seconds())
+
+	return err
+}
+
 func (m *machineProvider) finalizeRemoval(details *machineDetails) {
 	for {
 		err := m.removeMachine(details)
@@ -513,7 +496,9 @@ func (m *machineProvider) Acquire(config *common.RunnerConfig) (common.ExecutorD
 
 	// If we have a free machines we can process a build
 	if config.Machine.GetIdleCount() != 0 && machinesData.Idle == 0 {
-		err = errors.New("no free machines that can process builds")
+		err = &common.NoFreeExecutorError{
+			Message: "no free machines that can process builds",
+		}
 	}
 	return nil, err
 }
@@ -668,5 +653,25 @@ func newMachineProvider(name, executor string) *machineProvider {
 				},
 			},
 		),
+		stoppingHistogram: prometheus.NewHistogram(
+			prometheus.HistogramOpts{
+				Name:    "gitlab_runner_autoscaling_machine_stopping_duration_seconds",
+				Help:    "Histogram of machine stopping time.",
+				Buckets: []float64{1, 3, 5, 10, 30, 50, 60, 80, 90, 120},
+				ConstLabels: prometheus.Labels{
+					"executor": name,
+				},
+			},
+		),
+		removalHistogram: prometheus.NewHistogram(
+			prometheus.HistogramOpts{
+				Name:    "gitlab_runner_autoscaling_machine_removal_duration_seconds",
+				Help:    "Histogram of machine removal time.",
+				Buckets: []float64{1, 3, 5, 10, 30, 50, 60, 80, 90, 120},
+				ConstLabels: prometheus.Labels{
+					"executor": name,
+				},
+			},
+		),
 	}
 }
