@@ -7,12 +7,13 @@ import (
 
 	"gitlab.com/gitlab-org/gitlab-runner/common"
 	"gitlab.com/gitlab-org/gitlab-runner/executors"
+	"gitlab.com/gitlab-org/gitlab-runner/executors/vm"
 	prl "gitlab.com/gitlab-org/gitlab-runner/helpers/parallels"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/ssh"
 )
 
 type executor struct {
-	executors.AbstractExecutor
+	vm.Executor
 	vmName          string
 	sshCommand      ssh.Client
 	provisioned     bool
@@ -96,12 +97,7 @@ func (s *executor) restoreFromSnapshot() error {
 	return nil
 }
 
-func (s *executor) createVM() error {
-	baseImage := s.Config.Parallels.BaseName
-	if baseImage == "" {
-		return errors.New("missing Image setting from Parallels config")
-	}
-
+func (s *executor) createVM(baseImage string) error {
 	templateName := s.Config.Parallels.TemplateName
 	if templateName == "" {
 		templateName = baseImage + "-template"
@@ -117,7 +113,7 @@ func (s *executor) createVM() error {
 		s.Debugln("Creating template from VM", baseImage, "...")
 		err := prl.CreateTemplate(baseImage, templateName)
 		if err != nil {
-			return err
+			return fmt.Errorf("%w (image: %q)", err, baseImage)
 		}
 	}
 
@@ -135,7 +131,7 @@ func (s *executor) createVM() error {
 
 	// TODO: integration tests do fail on this due
 	// Unable to open new session in this virtual machine.
-	// Make sure the latest version of Parallels Tools is installed in this virtual machine and it has finished bootingg
+	// Make sure the latest version of Parallels Tools is installed in this virtual machine and it has finished booting
 	s.Debugln("Waiting for VM to start...")
 	err = prl.TryExec(s.vmName, 120, "exit", "0")
 	if err != nil {
@@ -184,9 +180,15 @@ func (s *executor) Prepare(options common.ExecutorPrepareOptions) error {
 		return err
 	}
 
+	var baseName string
+	baseName, err = s.Executor.GetBaseName(s.Config.Parallels.BaseName)
+	if err != nil {
+		return err
+	}
+
 	unregisterInvalidVM(s.vmName)
 
-	s.vmName = s.getVMName()
+	s.vmName = s.getVMName(baseName)
 
 	if s.Config.Parallels.DisableSnapshots && prl.Exist(s.vmName) {
 		s.Debugln("Deleting old VM...")
@@ -197,7 +199,7 @@ func (s *executor) Prepare(options common.ExecutorPrepareOptions) error {
 
 	if !prl.Exist(s.vmName) {
 		s.Println("Creating new VM...")
-		err = s.createVM()
+		err = s.createVM(baseName)
 		if err != nil {
 			return err
 		}
@@ -230,6 +232,10 @@ func (s *executor) printVersion() error {
 }
 
 func (s *executor) validateConfig() error {
+	if s.Config.Parallels.BaseName == "" {
+		return errors.New("missing BaseName setting from Parallels configuration")
+	}
+
 	if s.BuildShell.PassFile {
 		return errors.New("parallels doesn't support shells that require script file")
 	}
@@ -242,10 +248,7 @@ func (s *executor) validateConfig() error {
 		return errors.New("missing Parallels configuration")
 	}
 
-	if s.Config.Parallels.BaseName == "" {
-		return errors.New("missing BaseName setting from Parallels config")
-	}
-	return nil
+	return s.ValidateAllowedImages(s.Config.Parallels.AllowedImages)
 }
 
 func (s *executor) tryRestoreFromSnapshot() {
@@ -261,14 +264,14 @@ func (s *executor) tryRestoreFromSnapshot() {
 	}
 }
 
-func (s *executor) getVMName() string {
+func (s *executor) getVMName(baseName string) string {
 	if s.Config.Parallels.DisableSnapshots {
-		return s.Config.Parallels.BaseName + "-" + s.Build.ProjectUniqueName()
+		return baseName + "-" + s.Build.ProjectUniqueName()
 	}
 
 	return fmt.Sprintf(
 		"%s-runner-%s-concurrent-%d",
-		s.Config.Parallels.BaseName,
+		baseName,
 		s.Build.Runner.ShortDescription(),
 		s.Build.RunnerID,
 	)
@@ -393,8 +396,10 @@ func init() {
 
 	creator := func() common.Executor {
 		return &executor{
-			AbstractExecutor: executors.AbstractExecutor{
-				ExecutorOptions: options,
+			Executor: vm.Executor{
+				AbstractExecutor: executors.AbstractExecutor{
+					ExecutorOptions: options,
+				},
 			},
 		}
 	}
