@@ -14,6 +14,7 @@ import (
 
 	_ "gitlab.com/gitlab-org/gitlab-runner/cache/test"
 	"gitlab.com/gitlab-org/gitlab-runner/common"
+	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 	"gitlab.com/gitlab-org/gitlab-runner/helpers/tls"
 )
 
@@ -545,6 +546,128 @@ func TestWriteWritingArchiveCacheOnFailure(t *testing.T) {
 	}
 }
 
+func TestAbstractShell_writeCleanupBuildDirectoryScript(t *testing.T) {
+	testCases := []struct {
+		name                 string
+		buildDir             string
+		gitStrategy          string
+		gitCleanFlags        string
+		gitSubmoduleStrategy string
+		setupExpectations    func(*MockShellWriter)
+	}{
+		{
+			name:        "cloned repository",
+			buildDir:    "build/dir",
+			gitStrategy: "clone",
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("RmDir", "build/dir")
+			},
+		},
+		{
+			name:        "no git strategy",
+			buildDir:    "build/dir",
+			gitStrategy: "none",
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("Noticef", "Skipping build directory cleanup step")
+			},
+		},
+		{
+			name:        "git fetch strategy",
+			buildDir:    "some/dir/to/the/repo",
+			gitStrategy: "fetch",
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("Cd", "some/dir/to/the/repo")
+				m.On("Command", "git", "clean", "-ffdx")
+				m.On("Command", "git", "reset", "--hard")
+			},
+		},
+		{
+			name:          "git fetch with git clean flags",
+			buildDir:      "/build/dir/for/project",
+			gitStrategy:   "fetch",
+			gitCleanFlags: "-x -d -f",
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("Cd", "/build/dir/for/project")
+				m.On("Command", "git", "clean", "-x", "-d", "-f")
+				m.On("Command", "git", "reset", "--hard")
+			},
+		},
+		{
+			name:                 "git fetch with recursive submodule strategy",
+			buildDir:             "/dir/for/project",
+			gitStrategy:          "fetch",
+			gitCleanFlags:        "-n -q",
+			gitSubmoduleStrategy: "recursive",
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("Cd", "/dir/for/project")
+				m.On("Command", "git", "clean", "-n", "-q")
+				m.On("Command", "git", "reset", "--hard")
+				m.On("Command", "git", "submodule", "foreach", "--recursive", "git", "clean", "-n", "-q")
+				m.On("Command", "git", "submodule", "foreach", "--recursive", "git", "reset", "--hard")
+			},
+		},
+		{
+			name:                 "git fetch with normal submodule strategy",
+			buildDir:             "/dir/for/project",
+			gitStrategy:          "fetch",
+			gitCleanFlags:        "-x",
+			gitSubmoduleStrategy: "normal",
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("Cd", "/dir/for/project")
+				m.On("Command", "git", "clean", "-x")
+				m.On("Command", "git", "reset", "--hard")
+				m.On("Command", "git", "submodule", "foreach", "git", "clean", "-x")
+				m.On("Command", "git", "submodule", "foreach", "git", "reset", "--hard")
+			},
+		},
+		{
+			name:        "invalid git strategy",
+			buildDir:    "/dir/for/project",
+			gitStrategy: "use-svn",
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("RmDir", "/dir/for/project")
+			},
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			info := common.ShellScriptInfo{
+				Build: &common.Build{
+					JobResponse: common.JobResponse{
+						Variables: common.JobVariables{
+							common.JobVariable{
+								Key:   "FF_ENABLE_CLEANUP",
+								Value: "true",
+							},
+							common.JobVariable{
+								Key:   "GIT_STRATEGY",
+								Value: tc.gitStrategy,
+							},
+							common.JobVariable{
+								Key:   "GIT_CLEAN_FLAGS",
+								Value: tc.gitCleanFlags,
+							},
+							common.JobVariable{
+								Key:   "GIT_SUBMODULE_STRATEGY",
+								Value: tc.gitSubmoduleStrategy,
+							},
+						},
+					},
+					BuildDir: tc.buildDir,
+				},
+			}
+			mockShellWriter := &MockShellWriter{}
+			defer mockShellWriter.AssertExpectations(t)
+
+			tc.setupExpectations(mockShellWriter)
+			shell := AbstractShell{}
+
+			assert.NoError(t, shell.writeCleanupBuildDirectoryScript(mockShellWriter, info))
+		})
+	}
+}
+
 func TestGitCleanFlags(t *testing.T) {
 	tests := map[string]struct {
 		value string
@@ -733,10 +856,14 @@ func TestAbstractShell_writeSubmoduleUpdateCmd(t *testing.T) {
 					"Command",
 					append([]interface{}{"git", "submodule", "update", "--init"}, tc.ExpectedGitUpdateFlags...)...,
 				).Once()
-			mockWriter.
+			cleanCmd := mockWriter.
 				On("Command", append(expectedGitForEachArgsFn(), "git clean -ffxd")...).
 				Once()
-			mockWriter.On("Command", append(expectedGitForEachArgsFn(), "git reset --hard")...).Once()
+			mockWriter.On("Command", append(expectedGitForEachArgsFn(), "git reset --hard")...).
+				Run(func(args mock.Arguments) {
+					cleanCmd.Once()
+				}).
+				Once()
 			mockWriter.On("IfCmd", "git", "lfs", "version").Once()
 			mockWriter.On("Command", append(expectedGitForEachArgsFn(), "git lfs pull")...).Once()
 			mockWriter.On("EndIf").Once()
@@ -897,8 +1024,12 @@ func TestAbstractShell_writeSubmoduleUpdateCmdPath(t *testing.T) {
 			mockWriter.On("Noticef", mock.Anything).Once()
 			mockWriter.On("Command", submoduleCommand(test.paths, "sync")...).Once()
 			mockWriter.On("Command", submoduleCommand(test.paths, "update", "--init")...).Once()
-			mockWriter.On("Command", "git", "submodule", "foreach", "git clean -ffxd").Once()
-			mockWriter.On("Command", "git", "submodule", "foreach", "git reset --hard").Once()
+			cleanCmd := mockWriter.On("Command", "git", "submodule", "foreach", "git clean -ffxd").Once()
+			mockWriter.On("Command", "git", "submodule", "foreach", "git reset --hard").
+				Run(func(args mock.Arguments) {
+					cleanCmd.Once()
+				}).
+				Once()
 			mockWriter.On("IfCmd", "git", "lfs", "version").Once()
 			mockWriter.On("Command", "git", "submodule", "foreach", "git lfs pull").Once()
 			mockWriter.On("EndIf").Once()
@@ -987,6 +1118,7 @@ func TestWriteUserScript(t *testing.T) {
 					JobResponse: common.JobResponse{
 						Steps: tt.inputSteps,
 					},
+					Runner: &common.RunnerConfig{},
 				},
 				PostBuildScript: tt.postBuildScript,
 			}
@@ -1002,6 +1134,151 @@ func TestWriteUserScript(t *testing.T) {
 	}
 }
 
+func TestScriptSections(t *testing.T) {
+	tests := []struct {
+		inputSteps        common.Steps
+		setupExpectations func(*MockShellWriter)
+		featureFlagOn     bool
+		traceSections     bool
+	}{
+		{
+			featureFlagOn: true,
+			traceSections: true,
+			inputSteps: common.Steps{
+				common.Step{
+					Name:   common.StepNameScript,
+					Script: common.StepScript{"script 1", "script 2", "script 3"},
+				},
+			},
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("Variable", mock.Anything)
+				m.On("Cd", mock.AnythingOfType("string"))
+				m.On("SectionStart", mock.AnythingOfType("string"), "$ echo prebuild").Once()
+				m.On("SectionEnd", mock.AnythingOfType("string")).Once()
+				m.On("SectionStart", mock.AnythingOfType("string"), "$ script 1").Once()
+				m.On("SectionEnd", mock.AnythingOfType("string")).Once()
+				m.On("SectionStart", mock.AnythingOfType("string"), "$ script 2").Once()
+				m.On("SectionEnd", mock.AnythingOfType("string")).Once()
+				m.On("SectionStart", mock.AnythingOfType("string"), "$ script 3").Once()
+				m.On("SectionEnd", mock.AnythingOfType("string")).Once()
+				m.On("SectionStart", mock.AnythingOfType("string"), "$ echo postbuild").Once()
+				m.On("SectionEnd", mock.AnythingOfType("string")).Once()
+				m.On("Line", "echo prebuild").Once()
+				m.On("Line", "script 1").Once()
+				m.On("Line", "script 2").Once()
+				m.On("Line", "script 3").Once()
+				m.On("Line", "echo postbuild").Once()
+				m.On("CheckForErrors").Times(5)
+			},
+		},
+		{
+			featureFlagOn: false,
+			traceSections: false,
+			inputSteps: common.Steps{
+				common.Step{
+					Name:   common.StepNameScript,
+					Script: common.StepScript{"script 1", "script 2", "script 3"},
+				},
+			},
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("Variable", mock.Anything)
+				m.On("Cd", mock.AnythingOfType("string"))
+				m.On("Noticef", "$ %s", "echo prebuild").Once()
+				m.On("Noticef", "$ %s", "script 1").Once()
+				m.On("Noticef", "$ %s", "script 2").Once()
+				m.On("Noticef", "$ %s", "script 3").Once()
+				m.On("Noticef", "$ %s", "echo postbuild").Once()
+				m.On("Line", "echo prebuild").Once()
+				m.On("Line", "script 1").Once()
+				m.On("Line", "script 2").Once()
+				m.On("Line", "script 3").Once()
+				m.On("Line", "echo postbuild").Once()
+				m.On("CheckForErrors").Times(5)
+			},
+		},
+		{
+			featureFlagOn: true,
+			traceSections: false,
+			inputSteps: common.Steps{
+				common.Step{
+					Name:   common.StepNameScript,
+					Script: common.StepScript{"script 1", "script 2", "script 3"},
+				},
+			},
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("Variable", mock.Anything)
+				m.On("Cd", mock.AnythingOfType("string"))
+				m.On("Noticef", "$ %s", "echo prebuild").Once()
+				m.On("Noticef", "$ %s", "script 1").Once()
+				m.On("Noticef", "$ %s", "script 2").Once()
+				m.On("Noticef", "$ %s", "script 3").Once()
+				m.On("Noticef", "$ %s", "echo postbuild").Once()
+				m.On("Line", "echo prebuild").Once()
+				m.On("Line", "script 1").Once()
+				m.On("Line", "script 2").Once()
+				m.On("Line", "script 3").Once()
+				m.On("Line", "echo postbuild").Once()
+				m.On("CheckForErrors").Times(5)
+			},
+		},
+		{
+			featureFlagOn: false,
+			traceSections: true,
+			inputSteps: common.Steps{
+				common.Step{
+					Name:   common.StepNameScript,
+					Script: common.StepScript{"script 1", "script 2", "script 3"},
+				},
+			},
+			setupExpectations: func(m *MockShellWriter) {
+				m.On("Variable", mock.Anything)
+				m.On("Cd", mock.AnythingOfType("string"))
+				m.On("Noticef", "$ %s", "echo prebuild").Once()
+				m.On("Noticef", "$ %s", "script 1").Once()
+				m.On("Noticef", "$ %s", "script 2").Once()
+				m.On("Noticef", "$ %s", "script 3").Once()
+				m.On("Noticef", "$ %s", "echo postbuild").Once()
+				m.On("Line", "echo prebuild").Once()
+				m.On("Line", "script 1").Once()
+				m.On("Line", "script 2").Once()
+				m.On("Line", "script 3").Once()
+				m.On("Line", "echo postbuild").Once()
+				m.On("CheckForErrors").Times(5)
+			},
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(
+			fmt.Sprintf("feature flag %t, trace sections %t", tt.featureFlagOn, tt.traceSections),
+			func(t *testing.T) {
+				info := common.ShellScriptInfo{
+					PreBuildScript: "echo prebuild",
+					Build: &common.Build{
+						JobResponse: common.JobResponse{
+							Steps: tt.inputSteps,
+							Features: common.GitlabFeatures{
+								TraceSections: tt.traceSections,
+							},
+						},
+						Runner: &common.RunnerConfig{RunnerSettings: common.RunnerSettings{
+							FeatureFlags: map[string]bool{featureflags.ScriptSections: tt.featureFlagOn},
+						}},
+					},
+					PostBuildScript: "echo postbuild",
+				}
+				mockShellWriter := &MockShellWriter{}
+				defer mockShellWriter.AssertExpectations(t)
+
+				tt.setupExpectations(mockShellWriter)
+				shell := AbstractShell{}
+
+				assert.NoError(t, shell.writeUserScript(mockShellWriter, info, common.BuildStage("step_script")))
+			},
+		)
+	}
+}
+
 func TestSkipBuildStage(t *testing.T) {
 	stageTests := map[common.BuildStage]map[string]struct {
 		JobResponse common.JobResponse
@@ -1185,7 +1462,7 @@ func TestSkipBuildStage(t *testing.T) {
 				},
 			},
 		},
-		common.BuildStageCleanupFileVariables: {
+		common.BuildStageCleanup: {
 			"don't skip if file artifact defined": {
 				common.JobResponse{
 					Variables: common.JobVariables{
@@ -1261,6 +1538,7 @@ func TestAbstractShell_writeCleanupFileVariablesScript(t *testing.T) {
 					{Key: testVar4, Value: "test", File: false},
 				},
 			},
+			Runner: &common.RunnerConfig{},
 		},
 	}
 
@@ -1274,7 +1552,7 @@ func TestAbstractShell_writeCleanupFileVariablesScript(t *testing.T) {
 
 	shell := new(AbstractShell)
 
-	err := shell.writeCleanupFileVariablesScript(mockShellWriter, info)
+	err := shell.writeCleanupScript(mockShellWriter, info)
 	assert.NoError(t, err)
 }
 
@@ -1289,7 +1567,7 @@ func BenchmarkScriptStage(b *testing.B) {
 		common.BuildStageArchiveOnFailureCache,
 		common.BuildStageUploadOnSuccessArtifacts,
 		common.BuildStageUploadOnFailureArtifacts,
-		common.BuildStageCleanupFileVariables,
+		common.BuildStageCleanup,
 		common.BuildStage("step_release"),
 	}
 
